@@ -741,12 +741,13 @@ where
     type Err = Error;
 
     fn from_str(s: &str) -> Result<Descriptor<Pk>, Error> {
-        let desc_str = verify_checksum(s)?;
         // tr tree parsing has special code
-        if desc_str.starts_with("tr") {
-            let tr = Tr::from_str(desc_str)?;
-            Ok(Descriptor::Tr(tr))
+        // Tr::from_str will check the checksum
+        // match "tr(" to handle more extensibly
+        if s.starts_with("tr(") {
+            Ok(Descriptor::Tr(Tr::from_str(s)?))
         } else {
+            let desc_str = verify_checksum(s)?;
             let top = expression::Tree::from_str(desc_str)?;
             expression::FromTree::from_tree(&top)
         }
